@@ -9,7 +9,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2019 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -36,11 +36,10 @@
 #define CONST_8_TO_5_BITS 0.12156862745098
 #define CONST_8_TO_6_BITS 0.247058823529412
 
-static short colour_depth = 0;
-static long redmask, greenmask, bluemask;
+short colour_depth = 0;
+long redmask, greenmask, bluemask;
 
-static void set_up_gradient() {
-  int i;
+void set_up_gradient() {
 #ifdef BUILD_X11
   if (out_to_x.get(*state)) {
     colour_depth = DisplayPlanes(display, screen);
@@ -58,7 +57,7 @@ static void set_up_gradient() {
   redmask = 0;
   greenmask = 0;
   bluemask = 0;
-  for (i = (colour_depth / 3) - 1; i >= 0; i--) {
+  for (int i = (colour_depth / 3) - 1; i >= 0; i--) {
     redmask |= 1 << i;
     greenmask |= 1 << i;
     bluemask |= 1 << i;
@@ -85,16 +84,19 @@ unsigned int adjust_colours(unsigned int colour) {
 }
 
 /* this function returns the next colour between two colours for a gradient */
-unsigned long *do_gradient(int width, unsigned long first_colour,
-                           unsigned long last_colour) {
+std::unique_ptr<unsigned long[]> do_gradient(int width,
+                                             unsigned long first_colour,
+                                             unsigned long last_colour) {
   int red1, green1, blue1;           // first colour
   int red2, green2, blue2;           // last colour
   int reddiff, greendiff, bluediff;  // difference
   short redshift = (2 * colour_depth / 3 + colour_depth % 3);
   short greenshift = (colour_depth / 3);
-  auto *colours =
-      static_cast<unsigned long *>(malloc(width * sizeof(unsigned long)));
-  int i;
+
+  // Make sure the width is always at least 2
+  width = std::max(2, width);
+
+  std::unique_ptr<unsigned long[]> colours(new unsigned long[width]);
 
   if (colour_depth == 0) { set_up_gradient(); }
   red1 = (first_colour & redmask) >> redshift;
@@ -109,7 +111,7 @@ unsigned long *do_gradient(int width, unsigned long first_colour,
 #ifdef HAVE_OPENMP
 #pragma omp parallel for schedule(dynamic, 10) shared(colours)
 #endif /* HAVE_OPENMP */
-  for (i = 0; i < width; i++) {
+  for (int i = 0; i < width; i++) {
     int red3 = 0, green3 = 0, blue3 = 0;  // colour components
 
     float factor = (static_cast<float>(i) / (width - 1));
